
#include "Arduino.h"

#define ACQUISITION 104
#define STOP_ACQUISITION 105
#define ADC_MR_TRIG1 (1<<1)
uint8_t nchannels;
#define MAX_NCHAN 2
uint8_t channels[MAX_NCHAN];
uint8_t gain[MAX_NCHAN];
uint8_t offset[MAX_NCHAN];
uint8_t channels_pins[8] ={7,6,5,4,3,2,1,0};
#define NBUF 0x10
#define NBUF_MASK 0xF // (NBUF-1)
#define BUF_SIZE 0x100
#define BUF_MASK 0xFF // (BUF_SIZE-1)
uint16_t buf[MAX_NCHAN][NBUF][BUF_SIZE];
volatile uint8_t compteur_buf,compteur_buf_transmis;
volatile uint16_t indice_buf;
uint32_t nblocs;
uint8_t sans_fin;
            
void config_adc() {
    int k;
    pmc_enable_periph_clk(ID_ADC);
    adc_init(ADC, VARIANT_MCK, ADC_FREQ_MAX, 0);
    adc_configure_timing(ADC, 0, ADC_SETTLING_TIME_3, 1);
    adc_set_resolution(ADC,ADC_12_BITS);
    NVIC_EnableIRQ (ADC_IRQn) ;   
    ADC->ADC_IDR=0xFFFFFFFF ;  
    ADC->ADC_CHDR = 0xFFFF; 
    ADC->ADC_CHSR = 0x0000; 
    ADC->ADC_WPMR &= ~0x1;
    ADC->ADC_CHER = 0x0000;
    ADC->ADC_SEQR1 = 0x00000000;
    ADC->ADC_CGR = 0x00000000;
    ADC->ADC_COR = 0x00000000;
    for (k=0; k<nchannels; k++) {
      ADC->ADC_SEQR1 |= ((channels[k]&0xF)<<(4*k));
      ADC->ADC_CHER |= (1<<k);
      if (gain[k]==2) ADC->ADC_CGR |= 2 << (2*k);
      if (gain[k]==4) ADC->ADC_CGR |= 3 << (2*k);
      if (offset[k]) ADC->ADC_COR |= 1 << k;
    }
    ADC->ADC_MR = (ADC->ADC_MR & 0xFFFFFFF0) | ADC_MR_TRIG1 | ADC_MR_TRGEN | ADC_MR_USEQ;
    ADC->ADC_IDR=~(1<<channels[0]); 
    ADC->ADC_IER=1<<channels[0];
}
            
void config_timer(uint32_t ticks) {
  uint32_t channel = 0;
  uint8_t clock = TC_CMR_TCCLKS_TIMER_CLOCK1; // horloge 84MHz/2=42 MHz
  pmc_enable_periph_clk (TC_INTERFACE_ID + 0*3+channel) ;
  TC_Configure(TC0,channel, TC_CMR_WAVE | TC_CMR_WAVSEL_UP_RC | TC_CMR_ACPA_CLEAR | TC_CMR_ACPC_SET |clock); 
  TC_SetRC(TC0,channel,ticks); 
  TC_SetRA(TC0,channel,ticks >> 1);
  TC_Start(TC0, channel);
}
            
void stop_timer() {
    uint32_t channel = 0;
    TC_Stop(TC0,channel);
    pmc_disable_periph_clk (TC_INTERFACE_ID + 0*3+channel);
}
            
void ADC_Handler() {
  int k;
  for (k=0; k<nchannels; k++) {
    buf[k][compteur_buf][indice_buf] = *(ADC->ADC_CDR+channels[k]);
  }
  indice_buf++;
  if (indice_buf == BUF_SIZE) { // uint16_t indice_buffer
    indice_buf = 0;
    compteur_buf = (compteur_buf+1)&NBUF_MASK;
  }
}
             
void setup() {
   SerialUSB.begin(115200);
   int i,j,k;
   for (i=0; i<NBUF; i++)
     for (j=0; j<BUF_SIZE; j++)
        for (k=0; k<MAX_NCHAN; k++) 
            buf[k][i][j] = 2048;
   compteur_buf = compteur_buf_transmis = 0;
}
             
void lecture_acquisition() {
    uint32_t c1,c2,c3,c4;
    uint32_t ticks;
    int k;
    while (SerialUSB.available()<1) {};
    nchannels = SerialUSB.read();
    while (SerialUSB.available()<nchannels) {};
    for (k=0; k<nchannels; k++) {
      if (k < MAX_NCHAN) channels[k] = channels_pins[SerialUSB.read()]; 
    }
    while (SerialUSB.available()<nchannels) {};
    for (k=0; k<nchannels; k++) {
      if (k < MAX_NCHAN) gain[k] = SerialUSB.read(); 
    }
    while (SerialUSB.available()<nchannels) {};
    for (k=0; k<nchannels; k++) {
      if (k < MAX_NCHAN) offset[k] = SerialUSB.read(); 
    }
    if (nchannels > MAX_NCHAN) nchannels = MAX_NCHAN;
    while (SerialUSB.available()<4) {};
    c1 = SerialUSB.read();
    c2 = SerialUSB.read();
    c3 = SerialUSB.read();
    c4 = SerialUSB.read();
    ticks = ((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
    while (SerialUSB.available()<4) {};
    c1 = SerialUSB.read();
    c2 = SerialUSB.read();
    c3 = SerialUSB.read();
    c4 = SerialUSB.read();
    nblocs = ((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
    if (nblocs==0) {
        sans_fin = 1;
        nblocs = 1;
    }
    else sans_fin = 0;
    compteur_buf=compteur_buf_transmis=0;
    indice_buf = 0;
    config_adc();
    config_timer(ticks);
}

             
void stop_acquisition() {
    ADC->ADC_MR &= ~(0x1);
    stop_timer();
    compteur_buf=compteur_buf_transmis=0;
}
             
void lecture_serie() {
   char com;
   if (SerialUSB.available()>0) {
        com = SerialUSB.read();
        if (com==ACQUISITION) lecture_acquisition();
        if (com==STOP_ACQUISITION) stop_acquisition();
   }
}
             
void loop() {
    int k;
    if ((compteur_buf_transmis!=compteur_buf)&&(nblocs>0)) {
        for (k=0; k<nchannels; k++) SerialUSB.write((uint8_t *)buf[k][compteur_buf_transmis],BUF_SIZE*2);
        compteur_buf_transmis=(compteur_buf_transmis+1)&NBUF_MASK;
        if (sans_fin==0) {
            nblocs--;
            if (nblocs==0) stop_acquisition();
        }
    }
    else lecture_serie();
}
             